==> decoupled_kit-8.x-1.0-rc2.tar.gz: decoupled_kit/modules/decoupled_kit_breadcrumb/src/Controller/DefaultController.php <==
<?php

namespace Drupal\decoupled_kit_breadcrumb\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Url;

/**
 * Class DefaultController.
 */
class DefaultController extends ControllerBase {

  /**
   * Drupal\Core\DependencyInjection\ContainerInjectionInterface definition.
   *
   * @var \Drupal\Core\DependencyInjection\ContainerInjectionInterface
   */
  protected $decoupledKit;

  /**
   * Drupal\Core\Path\AliasManagerInterface definition.
   *
   * @var \Drupal\Core\Path\AliasManagerInterface
   */
  protected $pathAliasManager;

  /**
   * The path validator.
   *
   * @var \Drupal\Core\Path\PathValidatorInterface
   */
  protected $pathValidator;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->decoupledKit = $container->get('decoupled_kit');
    $instance->pathAliasManager = $container->get('path.alias_manager');
    $instance->pathValidator = $container->get('path.validator');
    return $instance;
  }

  /**
   * Get breadcrumb data for current page.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request object.
   *
   * @return string
   *   Return JSON.
   */
  public function index(Request $request) {
    $path = $this->decoupledKit->checkPath($request);
    $breadcrumbs = [];

    $frontpage_items = [
      'link' => '/',
      'title' => $this->t('Home'),
    ];

    $config = $this->config('decoupled_kit_breadcrumb.config');

    if ($path == '/') {
      $breadcrumbs[] = $frontpage_items;
    }
    else {
      $pathPatterns = [];
      $configPathPatterns = trim($config->get('path_patterns'));
      if (!empty($configPathPatterns)) {
        $pathPatterns = explode("\n", $configPathPatterns);
      }

      // Front page part.
      if ($config->get('need_front')) {
        $breadcrumbs[] = $frontpage_items;
      }

      // Main part.
      $main = $this->getBreadcrumbs($path, $pathPatterns);
      if (!empty($main)) {
        $breadcrumbs[] = $main;
      }

      // Title part.
      if ($config->get('need_title')) {
        $title = $this->getTitle($path, $pathPatterns);
        if (!empty($title)) {
          if (is_array($title)) {
            foreach ($title as $value) {
              $breadcrumbs[] = $value;
            }
          }
          else {
            $breadcrumbs[] = $title;
          }
        }
      }
    }

    // Check for valid links.
    if ($config->get('need_url_validate')) {
      $breadcrumbs = $this->checkLinks($breadcrumbs);
    }

    $array = [
      'path' => $path,
      'data' => $breadcrumbs,
    ];

    return new JsonResponse($array);
  }

  /**
   * Get breadcrumb title.
   *
   * @param string $path
   *   Input path.
   * @param array $pathPatterns
   *   Patterns from config.
   *
   * @return array|bool
   *   Title array or null.
   */
  protected function getTitle($path, array $pathPatterns) {
    $internal_path = $this->pathAliasManager->getPathByAlias($path);
    if ($internal_path && $internal_path != $path) {
      $entity = $this->decoupledKit->getEntityFromPath($internal_path);
      if (!empty($entity)) {
        $res = [];
        $title = NULL;
        $entity_type = $entity->getEntityTypeId();
        switch ($entity_type) {
          case 'taxonomy_term':
            $title = $entity->getName();

            $entityManager = $this->entityTypeManager()->getStorage($entity_type);
            $id = $entity->id();
            $parents = $entityManager->loadAllParents($id);
            if (!empty($parents)) {
              foreach (array_reverse($parents) as $term) {
                $term_alias = Url::fromRoute('entity.taxonomy_term.canonical', ['taxonomy_term' => $term->id()])->toString();

                if ($term_alias != $path) {
                  $res[] = [
                    'link' => $term_alias,
                    'title' => $term->label(),
                  ];
                }
              }
            }
            break;

          case 'node':
            $title = $entity->getTitle();

          default:
            break;
        }

        if (!$title) {
          return NULL;
        }

        $res[] = ['title' => $title];
        return $res;
      }
    }
    else {
      if (!empty($pathPatterns)) {
        // Get title from path patterns.
        foreach ($pathPatterns as $value) {
          list(, $link, $title) = explode('|', $value);
          $link = $this->decoupledKit->canonicalPath($link);

          if ($path == $link) {
            $res = [];
            $res[] = ['title' => trim($title)];
            return $res;
          }
        }
      }
    }

    return NULL;
  }

  /**
   * Get main part.
   *
   * @param string $path
   *   Input path.
   * @param array $pathPatterns
   *   Patterns from config.
   *
   * @return array|bool
   *   Main part of breadcrumb or NULL.
   */
  protected function getBreadcrumbs($path, array $pathPatterns) {
    $parts = explode('/', $path);
    $type = $parts[1];

    if (!empty($pathPatterns)) {
      // Find in path.
      foreach ($pathPatterns as $value) {
        list($p_type, $link, $title) = explode('|', $value);
        if (trim($p_type) == $type) {
          return [
            'link' => $this->decoupledKit->canonicalPath($link),
            'title' => trim($title),
          ];
        }
      }
    }

    // If not found - find in entities by page alias.
    $internal_path = $this->pathAliasManager->getPathByAlias($path);
    $entity = $this->decoupledKit->getEntityFromPath($internal_path);
    if (!empty($entity)) {
      $entity_bundle = $entity->bundle();
      if (!empty($entity_bundle)) {
        if (empty($pathPatterns)) {
          foreach ($pathPatterns as $value) {
            list($p_type, $link, $title) = explode('|', $value);
            $link = $this->decoupledKit->canonicalPath($link);

            if ($p_type == $entity_bundle) {
              return [
                'link' => $this->decoupledKit->canonicalPath($link),
                'title' => trim($title),
              ];
            }
          }
        }

        // If empty pathPatterns or not found
        // get data from entity.
        return [
          'link' => $this->decoupledKit->canonicalPath($entity_bundle),
          'title' => $entity_bundle,
        ];
      }
    }

    // If not found.
    return NULL;
  }

  /**
   * Check for valid links.
   *
   * @param array $breadcrumbs
   *   Breadcrumbs array.
   *
   * @return array
   *   Breadcrumbs array with valid links.
   */
  protected function checkLinks(array $breadcrumbs) {
    $res = [];
    foreach ($breadcrumbs as $breadcrumb) {
      $link = $breadcrumb['link'];
      if (empty($link) || $this->pathValidator->isValid($link)) {
        $res[] = $breadcrumb;
      }
    }
    return $res;
  }

}

==> decoupled_kit-8.x-1.1.tar.gz: decoupled_kit/modules/decoupled_kit_breadcrumb/src/Controller/DefaultController.php <==
<?php

namespace Drupal\decoupled_kit_breadcrumb\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Url;

/**
 * Class DefaultController.
 */
class DefaultController extends ControllerBase {

  /**
   * Drupal\Core\DependencyInjection\ContainerInjectionInterface definition.
   *
   * @var \Drupal\Core\DependencyInjection\ContainerInjectionInterface
   */
  protected $decoupledKit;

  /**
   * Drupal\Core\Path\AliasManagerInterface definition.
   *
   * @var \Drupal\Core\Path\AliasManagerInterface
   */
  protected $pathAliasManager;

  /**
   * The path validator.
   *
   * @var \Drupal\Core\Path\PathValidatorInterface
   */
  protected $pathValidator;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->decoupledKit = $container->get('decoupled_kit');
    $instance->pathAliasManager = $container->get('path.alias_manager');
    $instance->pathValidator = $container->get('path.validator');
    return $instance;
  }

  /**
   * Get breadcrumb data for current page.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Return breadcrumbs.
   */
  public function index(Request $request) {
    $path = $this->decoupledKit->checkPath($request);
    $breadcrumbs = [];

    $frontpage_items = [
      'link' => '/',
      'title' => $this->t('Home'),
    ];

    $config = $this->config('decoupled_kit_breadcrumb.config');

    if ($path == '/') {
      $breadcrumbs[] = $frontpage_items;
    }
    else {
      $pathPatterns = [];
      $configPathPatterns = trim($config->get('path_patterns'));
      if (!empty($configPathPatterns)) {
        $pathPatterns = explode("\n", $configPathPatterns);
      }

      // Front page part.
      if ($config->get('need_front')) {
        $breadcrumbs[] = $frontpage_items;
      }

      // Main part.
      $main = $this->getBreadcrumbs($path, $pathPatterns);
      if (!empty($main)) {
        foreach ($main as $value) {
          $breadcrumbs[] = $value;
        }
      }

      // Title part.
      if ($config->get('need_title')) {
        $title = $this->getTitle($path, $pathPatterns);
        if (!empty($title)) {
          if (is_array($title)) {
            foreach ($title as $value) {
              $breadcrumbs[] = $value;
            }
          }
          else {
            $breadcrumbs[] = $title;
          }
        }
      }
    }

    // Check for valid links.
    if ($config->get('need_url_validate')) {
      $breadcrumbs = $this->checkLinks($breadcrumbs);
    }

    $array = [
      'path' => $path,
      'data' => $breadcrumbs,
    ];

    return new JsonResponse($array);
  }

  /**
   * Get breadcrumb title.
   *
   * @param string $path
   *   Input path.
   * @param array $pathPatterns
   *   Patterns from config.
   *
   * @return array|bool
   *   Title array or null.
   */
  protected function getTitle($path, array $pathPatterns) {
    $internal_path = $this->pathAliasManager->getPathByAlias($path);
    if ($internal_path && $internal_path != $path) {
      $entity = $this->decoupledKit->getEntityFromPath($internal_path);
      if (!empty($entity)) {
        $res = [];
        $title = NULL;
        $entity_type = $entity->getEntityTypeId();
        switch ($entity_type) {
          case 'taxonomy_term':
            $title = $entity->getName();

            $entityManager = $this->entityTypeManager()->getStorage($entity_type);
            $id = $entity->id();
            $parents = $entityManager->loadAllParents($id);
            if (!empty($parents)) {
              foreach (array_reverse($parents) as $term) {
                $term_alias = Url::fromRoute('entity.taxonomy_term.canonical', ['taxonomy_term' => $term->id()])->toString();

                if ($term_alias != $path) {
                  $res[] = [
                    'link' => $term_alias,
                    'title' => $term->label(),
                  ];
                }
              }
            }
            break;

          case 'node':
            $title = $entity->getTitle();

          default:
            break;
        }

        if (!$title) {
          return NULL;
        }

        $res[] = ['title' => $title];
        return $res;
      }
    }
    else {
      if (!empty($pathPatterns)) {
        // Get title from path patterns.
        foreach ($pathPatterns as $value) {
          list(, $link, $title) = explode('|', $value);
          $link = $this->decoupledKit->canonicalPath($link);

          if ($path == $link) {
            $res = [];
            $res[] = ['title' => trim($title)];
            return $res;
          }
        }
      }
    }

    return NULL;
  }

  /**
   * Get main part.
   *
   * @param string $path
   *   Input path.
   * @param array $pathPatterns
   *   Patterns from config.
   *
   * @return array|bool
   *   Main part of breadcrumb or NULL.
   */
  protected function getBreadcrumbs($path, array $pathPatterns) {
    $parts = explode('/', $path);

    // Remove first and last elements.
    array_shift($parts);
    array_pop($parts);

    if (empty($parts)) {
      return NULL;
    }

    $res = [];
    foreach ($parts as $type) {
      if (!empty($pathPatterns)) {
        foreach ($pathPatterns as $value) {
          list($p_type, $link, $title) = explode('|', $value);
          if (trim($p_type) == $type) {
            $res[] = [
              'link' => $this->decoupledKit->canonicalPath($link),
              'title' => trim($title),
            ];
            break;
          }
        }
      }
    }

    if (empty($res)) {
      // If not found - find in entities by page alias.
      $internal_path = $this->pathAliasManager->getPathByAlias($path);
      $entity = $this->decoupledKit->getEntityFromPath($internal_path);
      if (!empty($entity)) {
        $entity_bundle = $entity->bundle();
        if (!empty($entity_bundle)) {
          $entity_bundle_title = $entity->type->entity->label();
          return [
            'link' => $this->decoupledKit->canonicalPath($entity_bundle),
            'title' => $entity_bundle_title,
          ];
        }
      }
    }

    return $res;
  }

  /**
   * Check for valid links.
   *
   * @param array $breadcrumbs
   *   Breadcrumbs array.
   *
   * @return array
   *   Breadcrumbs array with valid links.
   */
  protected function checkLinks(array $breadcrumbs) {
    $res = [];
    foreach ($breadcrumbs as $breadcrumb) {
      $link = $breadcrumb['link'];
      if (empty($link) || $this->pathValidator->isValid($link)) {
        $res[] = $breadcrumb;
      }
    }
    return $res;
  }

}
